from django.apps import AppConfig


class Smev3Config(AppConfig):
    """Конфиг приложения Концентратор (СМЭВ 3)."""

    name = __package__
    label = 'smev3_v321'
    verbose_name = 'СМЭВ 3'

    def ready(self) -> None:
        """Выполняет доп. операции, когда приложение готово.

        Выполняет проверку обязательных плагинов:
            Контингент,

        Выполняет регистрацию сигналов, асинк. задач,
        исполнителей сервисов СМЭВ (AIO) и менеджеров изменений.

        """

        from kinder.plugins.helpers import check_required_plugins

        # Данные из контингента используются в сервисах Концентратора (СМЭВ 3).
        dependencies = [
            'kinder.plugins.contingent',
        ]
        check_required_plugins(self.name, dependencies)

        # Инициализирует асинк. задачи, обсерверы и сигналы.
        from .base import tasks
        from .esnsi import tasks
        from . import observer
        from . import signals

        from .executors import RepositoryExecutors
        from .application_request.executors import ApplicationRequestExecutor
        from .get_application_admission_request.executors import (
            GetApplicationAdmissionRequestExecutor)
        from .application_rejection_request.executors import (
            ApplicationRejectionRequestExecutor)
        from concentrator.smev3_v321.get_application_queue.executors import (
            GetApplicationQueueRequestExecutor)
        from concentrator.smev3_v321.get_application_request.executors import (
            GetApplicationRequestExecutor)
        from .cancel_request.executors import CancelRequestExecutor
        from .application_order_info_request.executors import (
            ApplicationOrderInfoRequestExecutor)
        from .get_application_queue_reason_request.executors import (
            GetApplicationQueueReasonRequestExecutor)
        from .application_admission_request.executors import (
            ApplicationAdmissionRequestExecutor)

        config = (
            ApplicationRequestExecutor,
            GetApplicationAdmissionRequestExecutor,
            ApplicationRejectionRequestExecutor,
            GetApplicationQueueRequestExecutor,
            GetApplicationRequestExecutor,
            CancelRequestExecutor,
            ApplicationOrderInfoRequestExecutor,
            GetApplicationQueueReasonRequestExecutor,
            ApplicationAdmissionRequestExecutor,)

        for executor in config:
            RepositoryExecutors.set_executor(executor)

        from .application_request.changes import (
            Smev3DeclarationPrivilegeChangeHelper,
            Smev3DeclarationUnitChangeHelper,
            Smev3DeclarationDocsChangeHelper,
            Smev3DeclarationChangeHelper,
            Smev3DelegateChangeHelper,
            Smev3ChildrenChangeHelper,
            Smev3StorageHelper,
        )

        Smev3StorageHelper.register_change_helper(
            'DeclarationPrivilege', Smev3DeclarationPrivilegeChangeHelper
        )
        Smev3StorageHelper.register_change_helper(
            'DeclarationUnit', Smev3DeclarationUnitChangeHelper
        )
        Smev3StorageHelper.register_change_helper(
            Smev3DeclarationDocsChangeHelper.NAME_MODEL,
            Smev3DeclarationDocsChangeHelper)
        Smev3StorageHelper.register_change_helper(
            'Declaration', Smev3DeclarationChangeHelper)
        Smev3StorageHelper.register_change_helper(
            'Delegate', Smev3DelegateChangeHelper)
        Smev3StorageHelper.register_change_helper(
            'Children', Smev3ChildrenChangeHelper)
